#!/bin/bash

#input_folder should already exist and contain all the sequence files in .fastq format.
input_folder="input_seqs"

# output_dir is where you would like the quality filtered data to be written - this folder should not already exits as it will be created by the script
output_folder="seqs_QF"

maxee="0.5"

echo ""
echo @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
echo QUALITY CONTROL 1: QUALITY FILTERING
echo @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
echo ""
mkdir ${output_folder}
echo ""

for filename in ${input_folder}/*.fastq
	do
		echo ""
		echo ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		echo input is:
		echo ${filename}
		echo ""
		echo Removing sequences with maxee of:
		echo ${maxee}
		echo""
		usearch8 -fastq_filter ${filename} -fastaout "${output_folder}/$(basename "$filename" .fastq)_QF.fasta" -fastq_maxee ${maxee}
		echo ""
		echo DONE
		echo ""
		echo ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	done

echo ""
echo DONE
echo ""
echo XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

input_folder2="seqs_QF"
output_folder2="seqs_QF_DEREP"

echo ""
echo @@@@@@@@@@@
echo DEREPLICATE
echo @@@@@@@@@@@
echo ""
mkdir ${output_folder2}
echo ""

for filename in ${input_folder2}/*.fasta
	do
		echo ""
		echo ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		echo FINDING UNIQUE SEQUENCES
		echo input is:
		echo ${filename}
		echo""
		usearch8 -derep_fulllength ${filename} -fastaout "${output_folder2}/$(basename "$filename" .fasta)_DEREP.fasta" -sizeout 
		echo ""
		echo DONE
		echo ""
		echo ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	done

echo ""
echo DONE
echo ""
echo XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


input_folder3="seqs_QF_DEREP"
output_folder3="seqs_QF_DEREP_CF"

# discard reads with clusters less than this %/100 of the total number of reads e.g. for 0.1% abundance_cutoff=0.001
# abundance_cutoff="0.01"

echo ""
echo @@@@@@@@@@@@@@@
echo COUNT SEQUENCES
echo @@@@@@@@@@@@@@@
mkdir ${output_folder3}
echo ""

for filename in ${input_folder3}/*.fasta
	do
		echo ""
		echo ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		echo COUNTING SEQUENCES
		echo input is:
		echo ${filename}
		echo NUMBER OF SEQUENCES =
	    grep ">" -c ${filename}
		grep ">" -c ${filename} > ${filename}_size.txt
		tail ${filename}_size.txt | awk '{print $1*0.01}' >> ${output_folder3}/$(basename "$filename" .fasta)_minsize.txt		
		rm -r ${filename}_size.txt
		echo ""
		echo DONE
		echo ""
		echo ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	done

echo ""
echo DONE
echo ""
echo XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

input_folder3="seqs_QF_DEREP"
output_folder3="seqs_QF_DEREP_CF"

echo ""
echo @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
echo QUALITY CONTROL 2 — CHIMERAS+DENOISE
echo @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
echo ""

for filename in ${input_folder3}/*.fasta
	do
		echo ""
		echo ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		echo REMOVING CHIMERAS AND DENOISING
		echo input is:
		echo ${filename}
		echo""
		/Users/17598877/usearch9 -unoise ${filename} -fastaout "${output_folder3}/$(basename "$filename" .fasta)_CF_DENOISE.fasta" -tabbedout "${output_folder3}/$(basename "$filename" .fasta)_CF_DENOISE.txt"
		echo ""
		echo DONE
		echo ""
		echo ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	done

echo ""
echo DONE
echo ""
echo XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


input_folder4="seqs_QF_DEREP_CF"
output_folder4="seqs_QF_DEREP_CF_AF"

echo ""
echo @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
echo QUALITY CONTROL 3 — ABUNDANCE FILTERING
echo @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
echo ""
mkdir ${output_folder4}
echo ""

for filename in ${input_folder4}/*.fasta
	do
		echo ""
		echo ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		echo input is:
		echo ${filename}
		echo""
		echo REMOVING UNIQUE READS WITH CLUSTERS CONTAINING LESS THAN:
		cat ${input_folder4}/$(basename "$filename" _CF_DENOISE.fasta)_minsize.txt
		usearch8 -sortbysize ${filename} -fastaout "${output_folder4}/$(basename "$filename" .fasta)_AF.fasta" -minsize $(cat "${input_folder4}/$(basename "$filename" _CF_DENOISE.fasta)_minsize.txt")
		echo ""
		echo DONE
		rm -r ${input_folder4}/$(basename "$filename" _CF_DENOISE.fasta)_minsize.txt
		echo ""
		echo DONE
		echo ""
		echo ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	done

echo ""
echo DONE
echo ""
echo XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


input5="seqs_QF_DEREP_CF_AF"
output5="seqs_QF_DEREP_CF_AF_TOBLAST"
input6="seqs_QF_DEREP_CF_AF_TOBLAST"

# Removes "=" for the BLAST application

echo ""
echo @@@@@@@@@@@@@@@@@@@@
echo FORMATTING FOR BLAST
echo @@@@@@@@@@@@@@@@@@@@
echo ""
mkdir ${output5}
for filename in ${input5}/*.fasta
	do
		echo Formatting
		echo ${filename}
		cp -v ${filename} ${output5}
	done
for filename in ${input6}/*.fasta
	do
		sed -i '' 's/=/;/g' ${filename}
		echo ""
		echo Complete
		echo ""
	done
echo ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
echo ""
echo ""
